//
//  YKShortcut.h
//  YoupiKey
//
//  Created by phupe on Sat Jan 20 2001.
//  Copyright (c) 2001 Philippe HUP. All rights reserved.
//


#import <Cocoa/Cocoa.h>
#import "YKConstant.h"

@interface YKShortcut : NSObject

{
@private
NSMutableDictionary *shortcutData;		//shortcut dictionary (data)
NSString *CutName;				//shortcut name
BOOL CutInMenu;					//shortcut in menu
BOOL CutHasTimer;				//shortcut programed
NSImage *CutIcon;				//shortcut icon
int CutKeycode;					//keycode of the keyboard equivalent
id CutPanel;

}

//*********************************************
//API needed to be implemented by derived class
//*********************************************

//called when the shortcut is created to initialize variables - call super first and return self
- (id)initWithDictionary:(NSDictionary *)theShortcut;
//called to create a new shortcut - call super first to get the standard dictionary and then add your keys
//return the dictionary
+ (NSDictionary *)newDataWithObject:(id)sender;
//called at startup to get the item in the shortcut menu - you can return a submenu
+ (NSMenuItem *)shortcutsMenuItem;
//return the name of the action performed by the shortcut - identical to the menu title
+ (NSString *)shortcutsTitle;
//return the data specific to the shortcut
- (NSDictionary *)dataValue;
//called when the shortcut needs to be executed (the keyboard shortcut is pressed or the menu item is selected)
//you have to check if it's valid before running
- (void)run;


//******************************************
//API optionaly implemented by derived class
//******************************************

//implemented if you want to control the ability to delete the shortcut - call super first
- (BOOL)canDelete;
//implemented if you want the name not to be modified - super return YES;
- (BOOL)canChangeName;
//implemented if you want the shortcut not executed by keyboard - super return YES
- (BOOL)canHaveKeystroke;
//implemented if you want the shortcut not executed by a timer - super return YES
- (BOOL)canHaveTimer;
//implemented if you want to control if the shortcut is valid (called at startup) - super return YES
- (BOOL)isValid;
//called when the user wants to modify an invalid shortcut - usually you call 'editName'
- (void)validate;
//called before the shortcut is displayed in the window to be modified
- (void)willModifyShortcutInPanel:(id)panel withAccessoryView:(NSView *)accessoryView;
//called when a key is pressed
- (BOOL)wantKeyEvent;
//send the key event
- (void)keyEvent:(NSEvent *)anEvent;
//called before the modification made in the shortcut are validate - return NO if spefific parameters aren't valid
- (BOOL)willValidateModify;
//called when modification is aborted
- (void)abortModify;
//return an array of pasteboard types the plugin accept to create a new shortcut
+ (NSArray *)pasteboardTypes;
//return YES if the plugin can create a shortcut with the pasteboard
+ (NSNumber *)acceptPasteboard:(NSPasteboard *)pboard;
//create a new shortcut with the pasteboard
+ (NSDictionary *)newDataWithPasteboard:(NSPasteboard *)pboard;
//called before the YoupiKey menu is displayed
- (void)menuWillDisplay;
//called to get the item that appeared in the YoupiKey menu
//implement it to set a special item (submenu for example) - call super first to know if the shortcut is in the menu
- (NSMenuItem *)getMenuItem;


//*********************************
//API to be called by derived class
//*********************************

//set the icon that appeared in the list and in the menu
- (void)setIconValue:(NSData *)shortcutIcon;
//set the name that appeared in the list and in the menu
- (void)setNameValue:(NSString *)shortcutName;
//set the keyboard shortcut (normally not needed to be called)
- (void)setKeystrokeValue:(NSDictionary *)keystroke;
//set if the shortcut is in menu
- (void)setInMenuValue:(BOOL)inMenu;
//set if the shortcut has a timer
- (void)setTimerValue:(BOOL)hasTimer;
//set the timetable of the shortcut
- (void)setTimeTableValue:(NSDictionary *)timeTable;
//call it if your shortcut is not valid and you want to know if user want to make it valid
- (void)isNotValid;
- (BOOL)wantValidate;
//called to resize the panel of the edited shortcut
- (void)panelSizeToFit;
- (void)panelSetCanChangeName:(BOOL)canChange;
- (void)panelSetCanHaveKeystroke:(BOOL)canHave;
- (void)panelSetCanHaveTimer:(BOOL)canHave;
//called to set a possible name
- (void)setPossibleNameValue:(NSString *)shortcutName;
//called to set a possible icon
- (void)setPossibleIconValue:(NSData *)shortcutIcon;


//********************************************
//API normaly not implemented by derived class
//********************************************

//on return the icon that appeared in the list and the menu for the shortcut
- (id)iconValue;
//on return the name that appeared in the list and the menu for the shortcut
- (id)nameValue;
- (BOOL)nameIsEqual:(NSString *)theName;
//on return the string of the keyboard shortcut that appeared in the list
- (id)keystrokeValue;
//on return the NSImage that appeared in the list when the shortcut is in the menu (a checkmark)
- (id)inMenuValue;
//on return the NSImage that appeared in the list when the shortcut has a timer (a checkmark)
- (id)timerValue;
//on return the timetable of the shortcut
- (NSDictionary *)timeTableValue;
//called to know if the shortcut has a special keycode
- (BOOL)keycodeIsEqual:(int)keycode;
//on return YES if the shortcut is now in the YoupiKey menu
- (BOOL)isInMenu;
//called when the shortcut needs to be executed by its timer - super call 'run'
- (void)programRun:(NSTimer *)timer;
//called when the shorcut is removed from the YoupiKey menu
- (void)removeMenuItem;
//called when the next timer is set
- (void)program;
//called to get the next time the timer will execute the shortcut
- (id)nextTime;
//called when the next timer is removed
- (void)unprogram;
//called before the application linked to the set the shortcut belongs to is launched
- (void)appSetBeforeLaunch;
//called after the application linked to the set the shortcut belongs to is launched
- (void)appSetAfterLaunch;
//called after the application linked to the set the shortcut belongs to is terminated
- (void)appSetAfterTerminate;
//on return the data of the shortcut
- (NSDictionary *)dictValue;

//pour Applescript
- (NSString *)name;
- (id)runCommand:(NSScriptCommand *)command;


@end
